@@ -11,12 +11,18 @@ typedef struct
       int xml_port;
       int interactive_port;
       int server_threads;
+      int umask;
       llist_entry *trusted_hosts;
       int debug_level;
       int should_setuid;
       char *setuid_username;
       char *rrd_rootdir;
+      char *carbon_server;
+      int carbon_port;
+      int carbon_timeout;
+      char *graphite_prefix;
       int scalable_mode;
+      int write_rrds;
       int all_trusted;
       int num_RRAs;
       char *RRAs[MAX_RRAS];
