@@ -1,4 +1,3 @@
-/* $Id: gmetad.c 2190 2010-01-08 15:47:40Z d_pocock $ */
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -14,6 +13,7 @@
 #include <apr_time.h>
 
 #include "daemon_init.h"
+#include "update_pidfile.h"
 
 #include "rrd_helpers.h"
 
@@ -152,7 +152,10 @@ sum_metrics(datum_t *key, datum_t *val, void *arg)
    if (do_sum)
       {
          tt = in_type_list(type, strlen(type));
-         if (!tt) return 0;
+         if (!tt) {
+            datum_free(hash_datum);
+            return 0;
+	 }
 
          /* We sum everything in double to properly combine integer sources
             (3.0) with float sources (3.1).  This also avoids wraparound
@@ -237,7 +240,10 @@ write_root_summary(datum_t *key, datum_t *val, void *arg)
 
    /* err_msg("Writing Overall Summary for metric %s (%s)", name, sum); */
 
-   /* Save the data to a round robin database */
+   /* Save the data to a rrd file unless write_rrds == off */
+	 if (gmetad_config.write_rrds == 0)
+	     return 0;
+
    rc = write_data_to_rrd( NULL, NULL, name, sum, num, 15, 0, metric->slope);
    if (rc)
       {
@@ -256,6 +262,7 @@ main ( int argc, char *argv[] )
    pthread_attr_t attr;
    int i, num_sources;
    uid_t gmetad_uid;
+   mode_t rrd_umask;
    char * gmetad_username;
    struct passwd *pw;
    char hostname[HOSTNAMESZ];
@@ -324,7 +331,8 @@ main ( int argc, char *argv[] )
    /* Debug level 1 is error output only, and no daemonizing. */
    if (!debug_level)
       {
-         daemon_init (argv[0], 0);
+         rrd_umask = c->umask;
+         daemon_init (argv[0], 0, rrd_umask);
       }
 
    if (args_info.pid_file_given)
@@ -444,7 +452,7 @@ main ( int argc, char *argv[] )
 
          /* Save them to RRD */
          hash_foreach(root.metric_summary, write_root_summary, NULL);
-         
+
          /* Remember our last run */
          last_metadata = apr_time_now();
       }
