@@ -141,6 +141,9 @@ metric_report_start(Generic_t *self, datum_t *key, client_t *client, void *arg)
    tn = client->now.tv_sec - metric->t0.tv_sec;
    if (tn<0) tn = 0;
 
+   if (metric->dmax && metric->dmax < tn)
+     return 0;
+
    rc=xml_print(client, "<METRIC NAME=\"%s\" VAL=\"%s\" TYPE=\"%s\" "
       "UNITS=\"%s\" TN=\"%u\" TMAX=\"%u\" DMAX=\"%u\" SLOPE=\"%s\" "
       "SOURCE=\"%s\">\n",
@@ -186,10 +189,10 @@ host_report_start(Generic_t *self, datum_t *key, client_t *client, void *arg)
 
    /* Note the hash key is the host's IP address. */
    rc = xml_print(client, "<HOST NAME=\"%s\" IP=\"%s\" REPORTED=\"%u\" "
-      "TN=\"%u\" TMAX=\"%u\" DMAX=\"%u\" LOCATION=\"%s\" GMOND_STARTED=\"%u\">\n",
+      "TN=\"%u\" TMAX=\"%u\" DMAX=\"%u\" LOCATION=\"%s\" GMOND_STARTED=\"%u\" TAGS=\"%s\">\n",
       name, getfield(host->strings, host->ip), host->reported, tn,
       host->tmax, host->dmax, getfield(host->strings, host->location),
-      host->started);
+		  host->started, getfield(host->strings, host->tags));
 
    return rc;
 }
@@ -418,6 +421,7 @@ process_path (client_t *client, char *path, datum_t *myroot, datum_t *key)
          q = strchr(p, '/');
          if (!q) q=pathend;
       
+         /* len is limited in size by REQUESTLEN through readline() */
          len = q-p;
          element = malloc(len + 1);
          if ( element == NULL )
