@@ -37,7 +37,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "file.h"
+#include "gm_file.h"
 #include "libmetrics.h"
 
 #include <apr_tables.h>
@@ -49,19 +49,7 @@
  */
 mmodule multicpu_module;
 
-/* Never changes */
-#ifndef BUFFSIZE
-#define BUFFSIZE 16384
-#endif
-
-typedef struct {
-  struct timeval last_read;
-  float thresh;
-  char *name;
-  char buffer[BUFFSIZE];
-} timely_file;
-
-static timely_file proc_stat    = { {0,0} , 1., "/proc/stat" };
+static timely_file proc_stat    = { {0,0} , 1., "/proc/stat", NULL, BUFFSIZE };
 
 struct cpu_util {
    g_val_t val;
@@ -88,36 +76,6 @@ static cpu_util *cpu_wio = NULL;
 static cpu_util *cpu_intr = NULL;
 static cpu_util *cpu_sintr = NULL;
 
-float timediff(const struct timeval *thistime, const struct timeval *lasttime)
-{
-  float diff;
-
-  diff = ((double) thistime->tv_sec * 1.0e6 +
-          (double) thistime->tv_usec -
-          (double) lasttime->tv_sec * 1.0e6 -
-          (double) lasttime->tv_usec) / 1.0e6;
-
-  return diff;
-}
-
-static char *update_file(timely_file *tf)
-{
-    int rval;
-    struct timeval now;
-    gettimeofday(&now, NULL);
-    if(timediff(&now,&tf->last_read) > tf->thresh) {
-        rval = slurpfile(tf->name, tf->buffer, BUFFSIZE);
-        if(rval == SYNAPSE_FAILURE) {
-            err_msg("update_file() got an error from slurpfile() reading %s",
-                    tf->name);
-            return (char *)SYNAPSE_FAILURE;
-        }
-        else 
-            tf->last_read = now;
-    }
-    return tf->buffer;
-}
-
 /*
  * A helper function to determine the number of cpustates in /proc/stat (MKN)
  * Count how many cpus there are in the system
